@@ -3,6 +3,11 @@ MAIN_BRUSH_REPLACE_TIME = 1080000
 SIDE_BRUSH_REPLACE_TIME = 720000
 FILTER_REPLACE_TIME = 540000
 SENSOR_DIRTY_REPLACE_TIME = 108000
+MOP_ROLLER_REPLACE_TIME = 1080000
+STRAINER_REPLACE_TIME = 540000
+CLEANING_BRUSH_REPLACE_TIME = 1080000
+DUST_COLLECTION_REPLACE_TIME = 81000
+FLOOR_CLEANER_REPLACE_TIME = 1080000
 
 
 ROBOROCK_V1 = "ROBOROCK.vacuum.v1"
@@ -18,11 +23,14 @@ ROBOROCK_T7 = "roborock.vacuum.a11"  # cn s7
 ROBOROCK_T7S = "roborock.vacuum.a14"
 ROBOROCK_T7SPLUS = "roborock.vacuum.a23"
 ROBOROCK_S7_MAXV = "roborock.vacuum.a27"
+ROBOROCK_S7_MAXV_ULTRA = "roborock.vacuum.a65"
 ROBOROCK_S7_PRO_ULTRA = "roborock.vacuum.a62"
 ROBOROCK_Q5 = "roborock.vacuum.a34"
-ROBOROCK_Q7 = "roborock.vacuum.a37"  # CHECK THIS
+ROBOROCK_Q5_PRO = "roborock.vacuum.a72"
+ROBOROCK_Q7 = "roborock.vacuum.a40"
 ROBOROCK_Q7_MAX = "roborock.vacuum.a38"
 ROBOROCK_Q7PLUS = "roborock.vacuum.a40"
+ROBOROCK_Q8_MAX = "roborock.vacuum.a73"
 ROBOROCK_G10S_PRO = "roborock.vacuum.a26"
 ROBOROCK_G10S = "roborock.vacuum.a46"
 ROBOROCK_G10 = "roborock.vacuum.a29"
@@ -36,24 +44,27 @@ ROBOROCK_S8_PRO_ULTRA = "roborock.vacuum.a70"
 ROBOROCK_S8 = "roborock.vacuum.a51"
 ROBOROCK_P10 = "roborock.vacuum.a75"
 
-SUPPORTED_VACUUMS = (
-    [  # These are the devices that show up when you add a device - more could be supported and just not show up
-        ROBOROCK_G10,
-        ROBOROCK_G10S_PRO,
-        ROBOROCK_Q5,
-        ROBOROCK_Q7,
-        ROBOROCK_Q7_MAX,
-        ROBOROCK_S4,
-        ROBOROCK_S5_MAX,
-        ROBOROCK_S6,
-        ROBOROCK_S6_MAXV,
-        ROBOROCK_S6_PURE,
-        ROBOROCK_S7_MAXV,
-        ROBOROCK_S8_PRO_ULTRA,
-        ROBOROCK_S8,
-        ROBOROCK_S4_MAX,
-        ROBOROCK_S7,
-        ROBOROCK_P10,
-        ROCKROBO_G10_SG,
-    ]
-)
+ROBOROCK_DYAD_AIR = "roborock.wetdryvac.a107"
+ROBOROCK_DYAD_PRO_COMBO = "roborock.wetdryvac.a83"
+ROBOROCK_DYAD_PRO = "roborock.wetdryvac.a56"
+
+# These are the devices that show up when you add a device - more could be supported and just not show up
+SUPPORTED_VACUUMS = [
+    ROBOROCK_G10,
+    ROBOROCK_G10S_PRO,
+    ROBOROCK_Q5,
+    ROBOROCK_Q7,
+    ROBOROCK_Q7_MAX,
+    ROBOROCK_S4,
+    ROBOROCK_S5_MAX,
+    ROBOROCK_S6,
+    ROBOROCK_S6_MAXV,
+    ROBOROCK_S6_PURE,
+    ROBOROCK_S7_MAXV,
+    ROBOROCK_S8_PRO_ULTRA,
+    ROBOROCK_S8,
+    ROBOROCK_S4_MAX,
+    ROBOROCK_S7,
+    ROBOROCK_P10,
+    ROCKROBO_G10_SG,
+]
